﻿using System;

using Microsoft.Azure.Management.Compute.Fluent;
using Microsoft.Azure.Management.Compute.Fluent.Models;
using Microsoft.Azure.Management.Fluent;
using Microsoft.Azure.Management.Network.Fluent;
using Microsoft.Azure.Management.ResourceManager.Fluent;
using Microsoft.Azure.Management.ResourceManager.Fluent.Authentication;
using Microsoft.Azure.Management.ResourceManager.Fluent.Core;

namespace TestProject
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            AzureCredentials credentials = SdkContext.AzureCredentialsFactory.FromFile("azureauth.properties");

            IAzure azure = Azure.Configure()
                .WithLogLevel(HttpLoggingDelegatingHandler.Level.Basic)
                .Authenticate(credentials)
                .WithDefaultSubscription();

            #region 리소스 그룹을 생성한다.

            Console.WriteLine("리소스 그룹 생성중...");

            string groupName = "myResourceGroup";
            string vmName    = "myVM";
            Region location  = Region.USWest;

            var resourceGroup = azure.ResourceGroups.Define(groupName)
                .WithRegion(location)
                .Create();

            #endregion
            #region 가용성 집합을 생성한다.

            Console.WriteLine("가용성 집합 생성중...");

            IAvailabilitySet availabilitySet = azure.AvailabilitySets.Define("myAVSet")
                .WithRegion(location)
                .WithExistingResourceGroup(groupName)
                .WithSku(AvailabilitySetSkuTypes.Managed)
                .Create();

            #endregion
            #region 공용 IP 주소를 생성한다.

            Console.WriteLine("공용 IP 주소 생성중...");

            IPublicIPAddress publicIPAddress = azure.PublicIPAddresses.Define("myPublicIP")
                .WithRegion(location)
                .WithExistingResourceGroup(groupName)
                .WithDynamicIP()
                .Create();

            #endregion 
            #region 가상 네트워크를 생성한다.

            Console.WriteLine("가상 네트워크 생성중...");

            INetwork network = azure.Networks.Define("myVNet")
                .WithRegion(location)
                .WithExistingResourceGroup(groupName)
                .WithAddressSpace("10.0.0.0/16")
                .WithSubnet("mySubnet", "10.0.0.0/24")
                .Create();
            
            #endregion
            #region 네트워크 인터페이스를 생성한다.

            Console.WriteLine("네트워크 인터페이스 생성중...");

            INetworkInterface networkInterface = azure.NetworkInterfaces.Define("myNIC")
                .WithRegion(location)
                .WithExistingResourceGroup(groupName)
                .WithExistingPrimaryNetwork(network)
                .WithSubnet("mySubnet")
                .WithPrimaryPrivateIPAddressDynamic()
                .WithExistingPrimaryPublicIPAddress(publicIPAddress)
                .Create();

            #endregion
            #region 가상 머신을 생성한다.

            Console.WriteLine("가상 머신 생성중...");

            bool useMarketplaceImage = true;

            if(useMarketplaceImage)
            {
                azure.VirtualMachines.Define(vmName)
                    .WithRegion(location)
                    .WithExistingResourceGroup(groupName)
                    .WithExistingPrimaryNetworkInterface(networkInterface)
                    .WithLatestWindowsImage("MicrosoftWindowsServer", "WindowsServer", "2012-R2-Datacenter")
                    .WithAdminUsername("azureuser")
                    .WithAdminPassword("Azure12345678")
                    .WithComputerName(vmName)
                    .WithExistingAvailabilitySet(availabilitySet)
                    .WithSize(VirtualMachineSizeTypes.StandardDS1)
                    .Create();
            }
            else // 마켓플레이스 이미지 대신 기존 디스크를 사용해 가신 머신을 생성하는 경우
            {
                IDisk managedDisk = azure.Disks.Define("myosdisk")
                    .WithRegion(location)
                    .WithExistingResourceGroup(groupName)
                    .WithWindowsFromVhd("https://mystorage.blob.core.windows.net/vhds/myosdisk.vhd")
                    .WithSizeInGB(128)
                    .WithSku(DiskSkuTypes.PremiumLRS)
                    .Create();
            
                azure.VirtualMachines.Define("myVM")
                    .WithRegion(location)
                    .WithExistingResourceGroup(groupName)
                    .WithExistingPrimaryNetworkInterface(networkInterface)
                    .WithSpecializedOSDisk(managedDisk, OperatingSystemTypes.Windows)
                    .WithExistingAvailabilitySet(availabilitySet)
                    .WithSize(VirtualMachineSizeTypes.StandardDS1)
                    .Create();
            }

            #endregion
            #region 가상 머신 정보를 출력한다.

            var vm = azure.VirtualMachines.GetByResourceGroup(groupName, vmName);

            Console.WriteLine("가상 머신 정보 조회...");
            Console.WriteLine("hardwareProfile");
            Console.WriteLine("    vmSize : " + vm.Size);
            Console.WriteLine("storageProfile");
            Console.WriteLine("    imageReference");
            Console.WriteLine("          publisher : " + vm.StorageProfile.ImageReference.Publisher);
            Console.WriteLine("          offer     : " + vm.StorageProfile.ImageReference.Offer    );
            Console.WriteLine("          sku       : " + vm.StorageProfile.ImageReference.Sku      );
            Console.WriteLine("          version   : " + vm.StorageProfile.ImageReference.Version  );
            Console.WriteLine("    osDisk");
            Console.WriteLine("          osType       : " + vm.StorageProfile.OsDisk.OsType      );
            Console.WriteLine("          name         : " + vm.StorageProfile.OsDisk.Name        );
            Console.WriteLine("          createOption : " + vm.StorageProfile.OsDisk.CreateOption);
            Console.WriteLine("          caching      : " + vm.StorageProfile.OsDisk.Caching     );
            Console.WriteLine("osProfile");
            Console.WriteLine("    computerName           : " + vm.OSProfile.ComputerName                                     );
            Console.WriteLine("    adminUsername          : " + vm.OSProfile.AdminUsername                                    );
            Console.WriteLine("    provisionVMAgent       : " + vm.OSProfile.WindowsConfiguration.ProvisionVMAgent.Value      );
            Console.WriteLine("    enableAutomaticUpdates : " + vm.OSProfile.WindowsConfiguration.EnableAutomaticUpdates.Value);
            Console.WriteLine("networkProfile");

            foreach(string nicID in vm.NetworkInterfaceIds)
            {
                Console.WriteLine("    networkInterface id : " + nicID);
            }

            Console.WriteLine("vmAgent");
            Console.WriteLine("    vmAgentVersion" + vm.InstanceView.VmAgent.VmAgentVersion);
            Console.WriteLine("      statuses");

            foreach(InstanceViewStatus status in vm.InstanceView.VmAgent.Statuses)
            {
                Console.WriteLine("    code          : " + status.Code         );
                Console.WriteLine("    level         : " + status.Level        );
                Console.WriteLine("    displayStatus : " + status.DisplayStatus);
                Console.WriteLine("    message       : " + status.Message      );
                Console.WriteLine("    time          : " + status.Time         );
            }

            Console.WriteLine("disks");

            foreach(DiskInstanceView disk in vm.InstanceView.Disks)
            {
                Console.WriteLine("  name : " + disk.Name);
                Console.WriteLine("  statuses");

                foreach(InstanceViewStatus status in disk.Statuses)
                {
                    Console.WriteLine("    code          : " + status.Code         );
                    Console.WriteLine("    level         : " + status.Level        );
                    Console.WriteLine("    displayStatus : " + status.DisplayStatus);
                    Console.WriteLine("    time          : " + status.Time         );
                }
            }

            Console.WriteLine("VM general status");
            Console.WriteLine("   provisioningStatus : " + vm.ProvisioningState);
            Console.WriteLine("   id                 : " + vm.Id               );
            Console.WriteLine("   name               : " + vm.Name             );
            Console.WriteLine("   type               : " + vm.Type             );
            Console.WriteLine("   location           : " + vm.Region           );
            Console.WriteLine("VM instance status");

            foreach(InstanceViewStatus status in vm.InstanceView.Statuses)
            {
                Console.WriteLine("    code          : " + status.Code         );
                Console.WriteLine("    level         : " + status.Level        );
                Console.WriteLine("    displayStatus : " + status.DisplayStatus);
            }

            #endregion

            Console.WriteLine("다음 단계를 진행하려면 엔터 키를 눌러 주시기 바랍니다.");
            Console.ReadLine();

            #region 가상 머신을 중지한다.

            Console.WriteLine("가상 머신 중지중...");

            vm.PowerOff();

            #endregion

            Console.WriteLine("다음 단계를 진행하려면 엔터 키를 눌러 주시기 바랍니다.");
            Console.ReadLine();

            #region 가상 머신을 시작한다.

            Console.WriteLine("가상 머신 시작중...");

            vm.Start();

            #endregion

            Console.WriteLine("다음 단계를 진행하려면 엔터 키를 눌러 주시기 바랍니다.");
            Console.ReadLine();

            #region 가상 머신의 크기를 조정한다.

            Console.WriteLine("가상 머신 크기 조정중...");

            vm.Update()
                .WithSize(VirtualMachineSizeTypes.StandardDS2)
                .Apply();

            #endregion

            Console.WriteLine("다음 단계를 진행하려면 엔터 키를 눌러 주시기 바랍니다.");
            Console.ReadLine();

            #region 가상 머신에 데이터 디스크를 추가한다.

            Console.WriteLine("가상 머신에 데이터 디스크 추가중...");

            vm.Update()
                .WithNewDataDisk(2, 0, CachingTypes.ReadWrite) 
                .Apply();

            #endregion

            Console.WriteLine("다음 단계를 진행하려면 엔터 키를 눌러 주시기 바랍니다.");
            Console.ReadLine();

            #region 가상 머신의 할당을 취소한다.

            Console.WriteLine("가상 머신 할당 취소중...");

            vm.Deallocate();

            #endregion

            Console.WriteLine("리소스 그룹을 삭제하려면 엔터 키를 눌러 주시기 바랍니다...");
            Console.ReadLine();

            #region 리소스 그룹을 삭제한다.

            azure.ResourceGroups.DeleteByName(groupName);

            #endregion
        }

        #endregion
    }
}